#!/usr/bin/perl
# importProfiles.pl script 
# import DiscoveryProfiles into DA and resolve SNMP profile and IP domain names
# 	reads the output created by exportProfiles.pl
#
# Prerequisites: tested with perl 5.18
#	typically, REST::Client and XML::LibXML modules must be installed (cpan)
#
#	v1.1	March 30, 2015
#			supports a mix of (multiple) snmpv1 and snmpv3 profiles

# perl importProfiles.pl IMDA:8581 discPrfExp.xml profileAssociations.csv 1
#
# DISCLAIMER
# This script is provided AS IS and no warranties are provided or made.  
# For more information refer to http://www.ca.com/us/legal/terms-for-user-sites.aspx
#

use strict;
use REST::Client;
use XML::LibXML;

my $num_args = $#ARGV + 1;

if ($num_args < 4) {
	if ($num_args == 3) {
	  push(@ARGV, '0');
	} else {
	  print "\nUsage: getProfiles.pl DA_host_and_port (e.g. IMDA:8581) import_file association_file \n";
	  exit;
	}
}
my $input=$ARGV[0];
my $filename=$ARGV[1];
my $assocfilename=$ARGV[2];
my $debug=$ARGV[3];						# debug options: 1 - output to outfile only, no POST; 2 - create DP with appended -2; 
my $outfilename = "out-" . $filename;		
# only for testing
  
open(EXPORT,">>".$outfilename)
  or die "error when opening $outfilename for output $!\n";
print EXPORT "<DiscoveryProfileList>\n";

open(ASSOC, $assocfilename);
my $discprofname;
my $discprofname_set;
my $snmpprofname;
my $snmpprofid;
my $domainname;
my $domainid;
my $domain_set;
my $snmp_valid;
my $snmpnum;
my @profiletableSNMP;
my @profiletableDOMAIN;

#	file format: Discovery, ID, Name, ID, Type
while (<ASSOC>) {
	chomp;
	if ($_ =~ /(.+?),.+?,(.*?),.*?,SNMPv.$/) {
		$discprofname = $1;	
		$snmpprofname = $2;	
		my @profrow;
		push @profrow, $discprofname;
		push @profrow, $snmpprofname;
		push @profiletableSNMP, \@profrow;
	}		
}
close (ASSOC);
for (my $i = 0; $i < scalar(@profiletableSNMP); $i++) {
#	print $profiletableSNMP[$i][0] . "," . $profiletableSNMP[$i][1] . ",SNMP\n";
} 

open(ASSOC, $assocfilename);
while (<ASSOC>) {
	chomp;
	if ($_ =~ /(.+?),.+?,(.*?),.*?,DOMAIN$/) {
		$discprofname = $1;	
		$domainname = $2;	
		my @profrow;
		push @profrow, $discprofname;
		push @profrow, $domainname;
		push @profiletableDOMAIN, \@profrow;
	}		
}
close (ASSOC);

for (my $i = 0; $i < scalar(@profiletableDOMAIN); $i++) {
#	print $profiletableDOMAIN[$i][0] . "," . $profiletableDOMAIN[$i][1] . ",DOMAIN\n";
} 

my @snmptable;										# SNMPv1/v2c/v3 profile table (DA id, name)
my @domaintable;									# ip domain table (DA id, name)

my $host = "http://$input";
my $client = REST::Client->new();
$client->setHost($host);
$client->addHeader('Content-Type', 'application/xml');

### process DA snmp profiles
##################################################################################################

$client->GET('/rest/profiles/snmpv1/');
# print $client->responseContent();

my $parser = XML::LibXML->new();
my $snmpprofilelist    = $parser->load_xml(string => $client->responseContent());

foreach my $snmpprof ($snmpprofilelist->findnodes('/SNMPv1ProfileList/SNMPv1Profile')) {
    my($title) = $snmpprof->findnodes('./CommunicationProfile/ProfileName');
    # print $title->to_literal, "\n";
	my @commrow;
	push @commrow, $title->to_literal;
    my($title) = $snmpprof->findnodes('./ID');
    # print $title->to_literal, "\n";
	push @commrow, $title->to_literal;
	push @commrow, "SNMPv1";
	push @snmptable, \@commrow;
}
$client->GET('/rest/profiles/snmpv3/');
# print $client->responseContent();

my $snmpprofilelist    = $parser->load_xml(string => $client->responseContent());

foreach my $snmpprof ($snmpprofilelist->findnodes('/SNMPv3ProfileList/SNMPv3Profile')) {
    my($title) = $snmpprof->findnodes('./CommunicationProfile/ProfileName');
    # print $title->to_literal, "\n";
	my @commrow;
	push @commrow, $title->to_literal;
    my($title) = $snmpprof->findnodes('./ID');
    # print $title->to_literal, "\n";
	push @commrow, $title->to_literal;
	push @commrow, "SNMPv3";
	push @snmptable, \@commrow;
}

# get IPDomain name/id into table
$client->GET('/rest/ipdomains');
# print $client->responseContent();
my $ipdomainlist    = $parser->load_xml(string => $client->responseContent());

foreach my $ipdomain ($ipdomainlist->findnodes('/IPDomainList/IPDomain')) {
    my($title) = $ipdomain->findnodes('./Name');
    # print $title->to_literal, "\n";
	my @domrow;														
	push @domrow, $title->to_literal;
    my($title) = $ipdomain->findnodes('./ID');
    # print $title->to_literal, "\n";
	push @domrow, $title->to_literal;
	push @domaintable, \@domrow;
}

# process discovery profile export  
my $xmlout;
my $parser = XML::LibXML->new();
my $doc    = $parser->parse_file($filename);
print "Importing ...\n";
print "DPname, Name, ID, valid, Type\n";
print "DPname --------------------------------------- POST: RC\n";
foreach my $dpinst ($doc->findnodes('/DiscoveryProfileList/DiscoveryProfile')) {
    my($title) = $dpinst->findnodes('./Item/Name/text()');
    # print $title->to_literal, "\n";
	$discprofname = $title->to_literal;
	
	# lookup profile name and get related  domain name
	for (my $i = 0; $i < scalar(@profiletableDOMAIN); $i++) {
		if ($profiletableDOMAIN[$i][0] eq $discprofname) {
			$domainname = $profiletableDOMAIN[$i][1];
		}
	}
	$domain_set = 0;
	$domainid = "";
	#lookup domain id, should always be present 
	for (my $i = 0; $i < scalar(@domaintable); $i++) {
		if ($domaintable[$i][0] eq $domainname) {
			$domainid = $domaintable[$i][1];
		}
	}
	if (length($domainid)) {
		$domain_set = 1;				# domain found
	}

	if ($debug == 2) {
		# for test only: append -2 to $discprofname (eg if you need to import in same DA)
		$discprofname_set = $discprofname . "-2";
	} else {
		$discprofname_set = $discprofname;
	}
	$title->setData($discprofname_set);
	print $discprofname_set . ", " . $domainname  . ", " . $domainid . ", " . $domain_set . ", " . "DOMAIN\n";
	
	
	# DELETE a number of nodes that are internal and will be recreated
	my($DP)  = $dpinst->findnodes("./RelatesTo");
    if (length $DP)  {
      my $XML = $DP->parentNode;
	  $XML->removeChild($DP);
	}
    my($DP)  = $dpinst->findnodes("./ID");
    my $XML = $DP->parentNode;
    $XML->removeChild($DP);

	my($DP)  = $dpinst->findnodes("./Item/CreateTime");
    my $XML = $DP->parentNode;
	$XML->removeChild($DP);

    my($DP)  = $dpinst->findnodes("./MostRecentInstance");
    if (length $DP)  {
      my $XML = $DP->parentNode;
	  $XML->removeChild($DP);
	}
    my($DP)  = $dpinst->findnodes("./IsAlso");
    if (length $DP)  {
      my $XML = $DP->parentNode;
 	  $XML->removeChild($DP);
	}

	# lookup profile name and if found position first related SNMP name
	my $profSNMPix = 0;
	while ( ($profSNMPix < scalar(@profiletableSNMP)) && ($profiletableSNMP[$profSNMPix][0] ne $discprofname)) {
		$profSNMPix++;
	}
	# update SNMP profile IDs
	$snmp_valid = 0;
	$snmpnum = 0;
	my $results = $dpinst->findnodes('./SNMPProfileIDList/SNMPProfileID/text()');
	foreach my $snmpprofile ($results->get_nodelist) {
		$snmpprofid = ""; 
		$snmpnum++;
		if (($profiletableSNMP[$profSNMPix][0] eq $discprofname) && ($profSNMPix < scalar(@profiletableSNMP))) {
			$snmpprofname = $profiletableSNMP[$profSNMPix][1];
			# print $discprofname . ", " . $snmpprofname . " SNMP before lookup" . "\n";
			$profSNMPix++;
			if (length($snmpprofname)) {
				# lookup snmp profile id, only if explicit profile is provided
				for (my $i = 0; $i < scalar(@snmptable); $i++) {
					if ($snmptable[$i][0] eq $snmpprofname) {
						$snmpprofid = $snmptable[$i][1];
					}
				}
				if (length($snmpprofid)) {
					$snmp_valid++;				# SNMP profile given and found
				}
			}
			else {
				$snmp_valid++;					# SNMP profile not in DiscProfile, ok
			}

			my($title) = $snmpprofile->findnodes('.');			# use same sequence in both dpinst and profile table
			if (length $title)  {
				$title->setData($snmpprofid);
			}
			print $discprofname_set . ", " . $snmpprofname . ", " . $snmpprofid . ", " . $snmp_valid . ", SNMP\n";
		}
	}

     my($DP)  = $dpinst->findnodes("./IPDomainMember/IPDomainID/text()");
     if (length $DP)  {
		$DP->setData($domainid);
	}
	if (($snmp_valid == $snmpnum) && ($domain_set)) {
		$xmlout = '<?xml version="1.0" encoding="utf-8"?>';
		$xmlout = $xmlout . $dpinst->toString;
		if ($debug == 1) {
			print $discprofname_set . " --------------------------------------- POST: not POSTed" . "\n";
		} else {
			$client->POST('/rest/tenant/1/discoveryprofiles/', $xmlout);					# create disc profile under default tenant
			print $discprofname_set . " --------------------------------------- POST: " .  $client->responseCode() . "\n";
		}
		print EXPORT $xmlout . "\n\n";
	} else {
		print $discprofname_set . " --------------------------------------- POST: not VALID" . "\n";
	}
}  
print EXPORT "</DiscoveryProfileList>\n";

close (EXPORT);
  
